<?php
/**
 * REST API Cart controller
 *
 * Handles requests to the /checkout endpoints
 *
 * @author   Rhen Zabel
 * @category API
 * @package  Checkout REST API for WooCommerce/API
 * @since    1.0.1
 * @version  1.1.2
 */

include_once WC_ABSPATH . 'includes/wc-cart-functions.php';
include_once WC_ABSPATH . 'includes/wc-notice-functions.php';

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


/**
 * REST API Checkout controller class.
 *
 * @package WooCommerce Checkout REST API/API
 */
class WC_REST_Cart_Controller {
	/**
	 * Endpoint namespace.
	 *
	 * @var string
	 */
	private $namespace = 'violet/v1';

	// constants
	const TRACKING_NUMBER_FLAG = 'tracking number';

	/**
	 * Register the routes for cart.
	 *
	 * @access public
	 */
	public function register_routes() {
		register_rest_route( $this->namespace, '/checkout/shipping_methods', array(
			'methods' => 'POST',
			'callback' => array( $this, 'get_cart_shipping_methods' ),
			'permission_callback' => function () { return $this->permissions_check(); }
		) );

		register_rest_route( $this->namespace, '/checkout/totals', array(
			'methods' => 'POST',
			'callback' => array( $this, 'get_cart_totals' ),
			'permission_callback' => function () { return $this->permissions_check(); }
		) );

		register_rest_route( $this->namespace, '/shipment/details', array(
			'methods' => 'POST',
			'callback' => array( $this, 'get_shipment_details' ),
			'permission_callback' => function () { return $this->permissions_check(); }
		) );

		register_rest_route( $this->namespace, '/store_profile', array(
			'methods' => 'GET',
			'callback' => array( $this, 'get_store_profile' ),
			'permission_callback' => function () { return $this->permissions_check(); }
		) );

		register_rest_route( $this->namespace, '/configuration', array(
			'methods' => 'POST',
			'callback' => array( $this, 'apply_configuration' ),
			'permission_callback' => function () { return $this->permissions_check(); }
		) );

		register_rest_route( $this->namespace, '/access_scope', array(
			'methods' => 'GET',
			'callback' => array( $this, 'get_access_scope' ),
			'permission_callback' => function () { return $this->permissions_check(); }
		) );
	}

	/**
	 * Check whether a given request has permission to access Violet endpoints
	 *
	 * @return boolean
	 */
	public function permissions_check() {
		$userId = $this->perform_basic_authentication();
		if ( $userId ) {
			return true;
		} else {
			return false;
		}
	}


	/**
	 * Get Access Scope
	 *
	 * @access  public
	 * @since   1.0.5
	 * @version 1.0.5
	 * @param   array $request
	 * @return  WP_REST_Response
	 */
	public function get_access_scope( WP_REST_Request $request ) {
		
		$consumer_key = trim( $_SERVER['PHP_AUTH_USER'] );

		// use key from query param if basic auth not used
		if ( ! $consumer_key ) {
			$consumer_key = trim( $_GET['consumer_key'] );
		}

		// Get user data.
		$user = $this->get_user_data_by_consumer_key( $consumer_key );
		
		$resData = array (
			'scope' => $user->permissions,
			'user_id' => $user->user_id,
			'plugin_version' => '1.1.2'
		);

		// return rates data
		$response = new WP_REST_Response( $resData );
		$response->set_status( 200 );
		return $response;
	}


	/**
	 * Apply Violet Configuration
	 *
	 * @access  public
	 * @since   1.0.5
	 * @version 1.0.5
	 * @param   array $request
	 * @return  WP_REST_Response
	 */
	public function apply_configuration( WP_REST_Request $request ) {
		
		if ( !empty( $request['violet_merchant_id'] ) ) {
			update_option( 'violet_merchant_id', $request['violet_merchant_id'] );
		}

		if ( !empty( $request['violet_shipment_module'] ) ) {
			update_option( 'violet_shipment_module', $request['violet_shipment_module'] );
		}

		if ( !empty( $request['violet_payouts_link'] ) ) {
			update_option( 'violet_payouts_link', $request['violet_payouts_link'] );
		} else {
			update_option( 'violet_payouts_link', null );
		}

		if ( !empty( $request['violet_connection_status'] ) ) {
			update_option( 'violet_connection_status', $request['violet_connection_status'] );
		}

		$resData = array (
			'configuration' => array (
				'violet_merchant_id' => esc_html( wp_unslash( get_option( 'violet_merchant_id' ) ) ),
				'violet_shipment_module' => esc_html( wp_unslash( get_option( 'violet_shipment_module' ) ) ),
				'violet_connection_status' => esc_html( wp_unslash( get_option( 'violet_connection_status' ) ) )
			)
		);

		// return rates data
		$response = new WP_REST_Response( $resData );
		$response->set_status( 200 );
		return $response;
	}


	/**
	 * Get Cart Shipping Methods
	 *
	 * @access  public
	 * @since   1.0.1
	 * @version 1.1.1
	 * @param   array $request
	 * @return  WP_REST_Response
	 */
	public function get_cart_shipping_methods( WP_REST_Request $request ) {
		// init session
		WC()->initialize_session();

		// include frontend functions
		WC()->frontend_includes();

		// init cart
		WC()->initialize_cart();

		// flag as paying customer
		WC()->cart->get_customer()->set_is_paying_customer( true );

		// handle shipping address
		$this->map_shipping_address( $request['shipping_address'], WC()->cart->get_customer() );

		// handle items in cart
		foreach ( $request['skus'] as $sku ) {
			$idParts = explode( '::', $sku['external_id'] );
			$productId = $idParts[0];
			$variationId = ( sizeof( $idParts ) > 1 ? $idParts[1] : '' );
			$quantity = $sku['quantity'];
			WC()->cart->add_to_cart( $productId, $quantity, $variationId );
		}

		// handle coupons in cart
		$this->apply_coupons($request['discounts']);

		WC()->shipping->calculate_shipping( $this->get_shipping_packages( WC()->cart->get_shipping_packages() ) );
    	$packages = WC()->shipping->packages;

		$shippingRates = [];
		try {
			foreach ( $packages as $package ) {
				foreach ( $package['rates'] as $key => $rate ) {
					$shippingRates[] = array (
						"label" => $rate->label,
						"id" => $rate->id,
						"method_id" => $rate->method_id,
						"price" => $rate->cost
					);
				}
			}
		} catch ( Exception $e ) { /* pass */ }

		$resData = array (
			'rates' => $shippingRates
		);

		// return rates data
		$response = new WP_REST_Response( $resData );
		$response->set_status( 200 );
		return $response;
	}


	/**
	 * Get Cart Totals
	 *
	 * @access  public
	 * @since   1.0.1
	 * @version 1.1.1
	 * @param   array $request
	 * @return  WP_REST_Response
	 */
	public function get_cart_totals( WP_REST_Request $request ) {
		// init session
		WC()->initialize_session();

		// include frontend functions
		WC()->frontend_includes();

		// init cart
		WC()->initialize_cart();

		// handle items in cart
		foreach ( $request['skus'] as $sku ) {
			$idParts = explode( '::', $sku['external_id'] );
			$productId = $idParts[0];
			$variationId = ( sizeof( $idParts ) > 1 ? $idParts[1] : '' );
			$quantity = $sku['quantity'];
			WC()->cart->add_to_cart ($productId, $quantity, $variationId );
		}

		// handle coupons in cart
		$this->apply_coupons($request['discounts']);

		// flag as paying customer
		WC()->cart->get_customer()->set_is_paying_customer( true );

		// handle shipping address
		$this->map_shipping_address( $request['shipping_address'], WC()->cart->get_customer() );

		// calculate shipping costs
		WC()->shipping->calculate_shipping( $this->get_shipping_packages( WC()->cart->get_shipping_packages() ) );
		$packages = WC()->shipping->packages;

		// collect shipping tax values
		$shippingTax = $this->calculate_shipping_tax( $packages, $request );

		// calculate totals
		WC()->cart->calculate_totals();

		$resData = array (
			'totals' => array (
				'sub_total' => WC()->cart->subtotal_ex_tax,
				'tax_total' => round(WC()->cart->tax_total, 2) + $shippingTax,
				'shipping_total' => WC()->cart->shipping_total,
				'discount_total' => WC()->cart->get_discount_total(),
				'total' => WC()->cart->total
			),
			'coupons' => WC()->cart->get_applied_coupons()
		);

		// return rates data
		$response = new WP_REST_Response( $resData );
		$response->set_status( 200 );
		return $response;
	}


	/**
	 * Get Shipment Details
	 *
	 * @access  public
	 * @since   1.0.2
	 * @version 1.0.5
	 * @param   array $request
	 * @return  WP_REST_Response
	 */
	public function get_shipment_details( WP_REST_Request $request ) {
		// init response
		$resData = array ( 'fulfillments' => array() );

		// get selected shipment module
		$shipmentModule = get_option( 'violet_shipment_module' );

		// allow for fallback in request
		if ( !$shipmentModule && isset ( $request['module'] ) )
			$shipmentModule = $request['module'];

		// get shipping data from postmeta, if exists
		$shipmentData = $this->get_shipment_data( $request['order_id'], $shipmentModule );
		if ( $shipmentData ) {
			$resData['fulfillments'] = $this->compose_shipment_details( $shipmentData, $shipmentModule, $request['order_id'] );
		}

		// return shipment data
		$response = new WP_REST_Response( $resData );
		$response->set_status( 200 );
		return $response;
	}

	/**
	 * Get Store Profile
	 *
	 * @access  public
	 * @since   1.0.5
	 * @version 1.0.5
	 * @return  WP_REST_Response
	 */
	public function get_store_profile( WP_REST_Request $request ) {

		// get base country
		$countriesObj = new WC_Countries();

		// init empty array
		$resData = array ( 
			'store_profile' => array (
				'admin_email' => get_option( 'admin_email' ),
				'site_name' => get_option( 'blogname' ),
				'site_url' => get_option( 'siteurl' ),
				'default_locale' => get_locale(),
				'base_currency' => get_woocommerce_currency(),
				'base_country' => $countriesObj->get_base_country(),
				'version' => ( defined('WC_VERSION') ? WC_VERSION : null)
			) 
		);

		// return shipment data
		$response = new WP_REST_Response( $resData );
		$response->set_status( 200 );
		return $response;
	}


	/**
	* Get Shipment Meta
	*
	* @access  private
	* @since   1.0.2
	* @version 1.0.5
	* @param   string $orderId
	* @param   string $shippingModule
	* @see "https://woocommerce.github.io/code-reference/namespaces/default.html#function_wc_get_order_notes"
	*/
	private function get_shipment_data( $orderId, $shippingModule ) {

		switch ( $shippingModule ) {
			case 'WC_SERVICES':
				$postMeta = get_post_meta( $orderId, 'wc_connect_labels', true );
				if ( $postMeta ) {
					return maybe_unserialize( $postMeta );
				} else {
					return false;
				}
			case 'SHIPSTATION':
				return $this->extract_shipment_from_notes($orderId, 'SHIPSTATION');
			case 'PRINTFUL':
					return $this->extract_shipment_from_notes($orderId, 'PRINTFUL');
			default:
				return false;
		}
	}


	/**
	* Extract Shipment from Order Notes
	*
	* - attempts to extract shipment data from an array of order notes
	* - currently limited to ShipStation
	* @access private
	* @since   1.0.3
	* @version 1.0.5
	* @param   string $orderId
	* @return  array $shipmentData
	*/
	private function extract_shipment_from_notes( $orderId, $shippingModule ) {
		$orderNotes = wc_get_order_notes(array('order_id' => $orderId));
		$fulfillmentData = array();

		if (is_null($orderNotes) || empty($orderNotes))
			return false;

		// iterate order notes
		foreach ($orderNotes as $orderNote) {
			$shipmentData = array();
			// check if order note contains tracking number
			if (stripos($orderNote->content, self::TRACKING_NUMBER_FLAG) !== false) {
				
				// get shipment carrier
				$shipmentData['carrier'] = $this->match_carrier_name($orderNote->content);

				// extract Shipstation tracking number
				if ($shippingModule === 'SHIPSTATION') {
					$shipmentData = $this->process_shipstation_shipping_note($orderNote, $shipmentData, $orderId);
				}

				// extract Printful tracking number
				if ($shippingModule === 'PRINTFUL') {
					$shipmentData = $this->process_printful_shipping_note($orderNote, $shipmentData, $orderId);
				}
				$fulfillmentData[] = $shipmentData;
			}
		}
		return $fulfillmentData;
	}


	/** Process Shipstation Shipping Note
	* 
	* - attempts to extract shipstation shipping data with line items
	* 
	* @access private
	* @since   1.0.5
	* @version 1.0.5
	* @param   string $orderNote
	* @param   string $shipmentData
	* @param   string $orderId
	* @return  array $shipmentData
	*/
	private function process_shipstation_shipping_note( $orderNote, $shipmentData, $orderId ) {
		$strParts = explode (self::TRACKING_NUMBER_FLAG, $orderNote->content);
		if (sizeof($strParts) === 2) {
			$shipmentData['tracking_number'] = trim($strParts[1], " .");
		}
		
		$contentParts = explode('shipped via', $orderNote->content);
		if (sizeof($contentParts) === 2) {
			$order = wc_get_order($orderId);
			$lineItemParts = explode(', ', $contentParts[0]);
			foreach ($lineItemParts as $lineItemPart) {
				$skuParts = explode(' x ', $lineItemPart);

				$extendedSkuName = trim($skuParts[0]);
				$quantity = trim($skuParts[1]);

				$skuNameParts = explode(' ', $extendedSkuName);
				$skuName = implode(' ', array_slice($skuNameParts, 0, sizeof($skuNameParts) - 1));

				// get order sku data
				$orderSku = $this->find_matching_order_sku_by_name($skuName, $order->get_items());

				$shipmentData['skus'][] = array(
					'name' => $skuName,
					'quantity' => trim($quantity),
					'external_id' => ($orderSku !== null ? $orderSku['external_id'] : null)
				);
			}
		}
		return $shipmentData;
	}


	/**
	 * Process Printful Shipping Note
	 * 
	 * - attempts to extract printful shipping data with line items
	 * 
	 * @access private
	 * @since   1.0.5
	 * @version 1.0.5
	 * @param   string $orderNote
	 * @param   string $shipmentData
	 * @param   string $orderId
	 * @return  array $shipmentData
	 */
	private function process_printful_shipping_note($orderNote, $shipmentData, $orderId) {
		$htmlDom = new DOMDocument;
		@$htmlDom->loadHTML($orderNote->content);
		$links = $htmlDom->getElementsByTagName('a');

		// iterate links to find tracking number
		foreach($links as $link){
			$shipmentData['tracking_number'] = trim($link->textContent);
			$shipmentData['tracking_url'] = trim($link->getAttribute('href'));
			break;
		}

		try {
			$order = wc_get_order($orderId);
			$brNodes = $htmlDom->getElementsByTagName('br');
			foreach ($brNodes as $brNode) {
				$nextNode = $brNode->nextSibling;

				// limit to text nodes
				if ($nextNode->nodeName == "#text") {
					// split name parts 
					$nameParts = explode(" - ", $nextNode->wholeText);
					$partsCount = count($nameParts);

					$quantity = 0;
					if ($partsCount > 1) {
						$quantity = rtrim($nameParts[$partsCount - 1], 'x');
					}

					// reform name without quantity
					$skuName = implode(' - ', array_slice($nameParts, 0, $partsCount - 1));

					// get order sku data
					$orderSku = $this->find_matching_order_sku_by_name($skuName, $order->get_items());

					$shipmentData['skus'][] = array(
						'name' => $skuName,
						'quantity' => trim($quantity),
						'external_id' => ($orderSku !== null ? $orderSku['external_id'] : null)
					);
				}
				// escape, end of line items list
				else if ($nextNode->nodeName == "b") {
					break;
				}
			}
		} catch ( Exception $e ) { /* pass */ }

		// get carrier service name
		$carrierService = $this->match_carrier_method($shipmentData['carrier'], $orderNote->content);
		if (!is_null($carrierService)) {
			$shipmentData['label'] = $carrierService;
		}

		return $shipmentData;
	}


	/**
	* Match Carrier Name
	*
	* - attempts to extract a known carrier name from the body of an order note
	* @access private
	* @since   1.0.3
	* @version 1.0.3
	* @param string $noteBody
	* @return string
	*/
	private function match_carrier_name($noteBody) {
		if (is_null($noteBody)) return "OTHER";
		if (stripos($noteBody, " usps")) return "USPS";
		if (stripos($noteBody, " fedex")) return "FEDEX";
		if (stripos($noteBody, " dhl")) return "DHL";
		if (stripos($noteBody, " ontrac")) return "ONTRAC";
		if (stripos($noteBody, " ups")) return "UPS";
		return "OTHER";
	}


	/**
	* Match Carrier Method
	*
	* @access private
	* @since   1.0.4
	* @version 1.0.5
	* @param string $carrier
	* @param string $noteBody
	* @return string
	*/
	private function match_carrier_method($carrier, $noteBody) {
		if (is_null($carrier) || is_null($noteBody)) return null;
		// handle USPS
		if ($carrier === 'USPS') {
			if (stripos($noteBody, "first class ma")) return "First Class Mail";
			if (stripos($noteBody, "first class pa")) return "First Class Package";
			if (stripos($noteBody, "priority mail ex")) return "Priority Mail Express";
			if (stripos($noteBody, "priority mail")) return "Priority Mail";
			if (stripos($noteBody, "retail gr")) return "Retail Ground";
			if (stripos($noteBody, "media mai")) return "Media Mail";
		}

		// handle UPS
		else if ($carrier === 'UPS') {
			if (stripos($noteBody, "next day air e")) return "Next Day Air Early";
			if (stripos($noteBody, "next day air s")) return "Next Day Air Saver";
			if (stripos($noteBody, "next day air")) return "Next Day Air";
			if (stripos($noteBody, "2nd day air a")) return "2nd Day Air A.M.";
			if (stripos($noteBody, "2nd day air")) return "2nd Day Air";
			if (stripos($noteBody, "3 day s")) return "3 Day Select";
			if (stripos($noteBody, "ground")) return "Ground";
		}

		// handle DHL
		else if ($carrier === 'DHL') {
			if (stripos($noteBody, "parcel exp")) return "Parcel Expedited Max";
			if (stripos($noteBody, "parcel pl")) return "Smartmail Parcel Plus";
			if (stripos($noteBody, "parcel")) return "Smartmail Parcel";
			if (stripos($noteBody, "sameday sp")) return "Sameday Sprintline";
			if (stripos($noteBody, "sameday jet")) return "Sameday Jetline";
			if (stripos($noteBody, "express ea")) return "Express Easy";
			if (stripos($noteBody, "express en")) return "Express Envelope";
			if (stripos($noteBody, "express wor")) return "Express Worldwide";
			if (stripos($noteBody, "express")) return "Express";
		}

		return null;
	}


	/**
	* Compose Shipment Details
	*
	* @access private
	* @since   1.0.2
	* @version 1.0.5
	* @param array $shipmentData
	* @param string $shippingModule
	* @return array $shipmentDetails
	*/
	private function compose_shipment_details( $shipmentData, $shippingModule, $orderId ) {
		$fulfillments = array();

		switch ( $shippingModule ) {
			case 'WC_SERVICES':
				foreach ($shipmentData as $shipment) {
					$shipmentDetails = array();
					$shipmentDetails['tracking_number'] = $shipment['tracking'];
					$shipmentDetails['label'] = $shipment['service_name'];
					$shipmentDetails['carrier'] = $shipment['carrier_id'];
					$shipmentDetails['price'] = $shipment['rate'];
					$shipmentDetails['id'] = $shipment['label_id'];
					
					$shipmentDetails['skus'] = array();
					if ( sizeof( $shipment['product_ids'] ) > 0 ) {
						$order = wc_get_order($orderId);
						foreach ($shipment['product_ids'] as $productId) {
							$orderLineItem = $this->find_matching_order_sku($productId, $order->get_items());
							if ($orderLineItem != null) {
								$shipmentDetails['skus'][] = $orderLineItem;
							}
						}
					}
					$fulfillments[] = $shipmentDetails;
				}

				break;
			case 'SHIPSTATION':
			case 'PRINTFUL':
				return $shipmentData;
		}
		return $fulfillments;
	}


	/**
	* Compose Shipment Details
	*
	* @access private
	* @since   1.0.5
	* @version 1.0.5
	* @param string $productId
	* @param array $orderItems
	* @return array $skuDetails
	*/
	private function find_matching_order_sku( $productId, $orderItems ) {
		foreach ( $orderItems as $itemId => $item ) {
			if ( !empty($item->get_variation_id()) && $item->get_variation_id() == $productId) {
				return array (
					'name' => $item->get_name(),
					'external_id' => $this->compose_external_item_id( $item )
				);
			} else if ($item->get_product_id() == $productId) {
				return array (
					'name' => $item->get_name(),
					'external_id' => $this->compose_external_item_id( $item )
				);
			}
		}
		return null;
	}


	/**
	* Compose Shipment Details
	*
	* @access private
	* @since   1.0.5
	* @version 1.0.5
	* @param string $productName
	* @param array $orderItems
	* @return array $skuDetails
	*/
	private function find_matching_order_sku_by_name( $productName, $orderItems ) {
		foreach ( $orderItems as $itemId => $item ) {
			if ( !empty($item->get_name()) && $item->get_name() == $productName) {
				return array (
					'name' => $item->get_name(),
					'external_id' => $this->compose_external_item_id( $item )
				);
			}
		}
		return null;
	}


	/**
	* Compose Shipment Details
	*
	* @access private
	* @since   1.0.5
	* @version 1.0.5
	* @param object $item
	* @return $externalId
	*/
	private function compose_external_item_id( $item ) {
		if ( !empty($item->get_variation_id()) ) {
			return $item->get_product_id() . ":" . $item->get_variation_id();
		} else {
			return $item->get_product_id();
		}
	}


	/**
	* Calculate Shipping Tax
	*
	* @access  private
	* @since   1.0.1
	* @version 1.0.1
	* @param   array $packages
	* @param   array $request
	*/
	private function calculate_shipping_tax( $packages, $request ) {
		$shippingTax = 0;
		try {
			foreach ( $packages as $package ) {
				foreach ( $package['rates'] as $key => $rate ) {
					if ( $key == $request['shipping_method']['external_id'] ) {
						if ( isset( $rate->taxes ) && !empty( $rate->taxes ) )
							foreach ( $rate->taxes as $tKey => $tRate )
								$shippingTax += bcdiv( $tRate, 1, 2 );
						break;
					}
				}
			}
		} catch ( Exception $e ) { /* pass */ }
		return $shippingTax;
	}


	/**
	* Filter Shipping Packages
	*
	* @access  private
	* @since   1.0.1
	* @version 1.0.1
	* @param   array $packages
	*/
	private function get_shipping_packages( $packages ) {
    return apply_filters( 'woocommerce_cart_shipping_packages', $packages );
	}


	/**
	* Map Shipping Address
	*
	* @access  private
	* @since   1.0.1
	* @version 1.0.1
	* @param  object $address
	* @param  object $customer
	*/
	private function map_shipping_address( $address, $customer ) {
		$customer->set_shipping_address_1( $address['address_1'] );
		if ( isset( $address['address_2'] ) ) {
			$customer->set_shipping_address_2( $address['address_2'] );
		}
		$customer->set_shipping_city( $address['city'] );
		$customer->set_shipping_state( $address['state'] );
		$customer->set_shipping_postcode( $address['postal_code'] );
		$customer->set_shipping_country( $address['country'] );
	}


	/**
	* Map Billing Address
	*
	* @access  private
	* @since   1.0.1
	* @version 1.0.1
	* @param  object $address
	* @param  object $customer
	*/
	private function map_billing_address( $address, $customer ) {
		$customer->set_billing_address_1( $address['address_1'] );
		if ( isset( $address['address_2'] ) ) {
			$customer->set_billing_address_2( $address['address_2'] );
		}
		$customer->set_billing_city( $address['city'] );
		$customer->set_billing_state( $address['state'] );
		$customer->set_billing_postcode( $address['postal_code'] );
		$customer->set_billing_country( $address['country'] );
	}

	/**
	 * Basic Authentication.
	 *
	 * SSL-encrypted requests are not subject to sniffing or man-in-the-middle
	 * attacks, so the request can be authenticated by simply looking up the user
	 * associated with the given consumer key and confirming the consumer secret
	 * provided is valid.
	 *
	 * @return int|bool
	 */
	private function perform_basic_authentication() {
		$consumer_key = trim( $_SERVER['PHP_AUTH_USER'] ); // WPCS: CSRF ok, sanitization ok.
		$consumer_secret = trim( $_SERVER['PHP_AUTH_PW'] ); // WPCS: CSRF ok, sanitization ok.

		// check for query param sourced keys if basic auth values are not provided
		if ( ! $consumer_key || ! $consumer_secret ) {
			$consumer_key = trim( $_GET['consumer_key'] );
			$consumer_secret = trim( $_GET['consumer_secret'] );
		}

		// Stop if don't have any key.
		if ( ! $consumer_key || ! $consumer_secret ) {
			return false;
		}

		// Get user data.
		$this->user = $this->get_user_data_by_consumer_key( $consumer_key );
		if ( empty( $this->user ) ) {
			return false;
		}

		// Validate user secret.
		if ( ! hash_equals( $this->user->consumer_secret, $consumer_secret ) ) { // @codingStandardsIgnoreLine
			return false;
		}

		return $this->user->user_id;
	}


	/**
	 * Return the user data for the given consumer_key.
	 *
	 * @param string $consumer_key Consumer key.
	 * @return array
	 */
	private function get_user_data_by_consumer_key( $consumer_key ) {
			global $wpdb;

			$consumer_key = wc_api_hash( sanitize_text_field( $consumer_key ) );
			$user         = $wpdb->get_row(
					$wpdb->prepare(
							"
					SELECT key_id, user_id, permissions, consumer_key, consumer_secret, nonces
					FROM {$wpdb->prefix}woocommerce_api_keys
					WHERE consumer_key = %s
			",
							$consumer_key
					)
			);

			return $user;
	}

	/**
	* Apply coupons
	*
	* @access  private
	* @since   1.1.0
	* @version 1.1.1
	* @param  object $discounts
	*/
	private function apply_coupons( $discounts ) {
		foreach ( $discounts as $discount ) {
			if ( $discount['type'] == 'CODE' ) {
				WC()->cart->apply_coupon($discount['code']);
			}
		}
	}
}